<?php

namespace Shqawe\BestUserInXTime\XF\Repository;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;

class BuserRepo extends Repository
{

    public function getBestThreadsWriter($no_days)
    {
        $totlalTime = 86400 * $no_days;
        $startTime  = \XF::$time - \XF::$time % $totlalTime;
        $endtime    = $startTime + $totlalTime;
        $db = \XF::db();
        $bestDetails = $db->fetchRow("
        SELECT user.*, COUNT(*) AS total
        FROM xf_thread AS thread , xf_user AS user
        WHERE thread.user_id = user.user_id
        AND thread.post_date BETWEEN $startTime AND $endtime
        GROUP BY thread.user_id ORDER BY total DESC LIMIT 1
        ");
        if($bestDetails)
        {
            $user = \XF::em()->findOne('XF:User', ['user_id' , $bestDetails['user_id']]);
            return $user;
        }

        return false;
    }

    public function getBestPostsWriter($no_days)
    {
        $totlalTime = 86400 * $no_days;
        $startTime  = \XF::$time - \XF::$time % $totlalTime;
        $endtime    = $startTime + $totlalTime;
        $db = \XF::db();
        $bestDetails = $db->fetchRow("
        SELECT user.*, COUNT(*) AS total
        FROM  xf_user AS user , xf_post AS post
        WHERE post.user_id = user.user_id  
        AND post.post_date BETWEEN $startTime AND $endtime
        GROUP BY post.user_id ORDER BY total DESC LIMIT 1
        ");
        if($bestDetails)
        {
            $user = \XF::em()->findOne('XF:User', ['user_id' , $bestDetails['user_id']]);
            return $user;
        }

        return false;
    }


    public function getPobularPost($no_days)
    {
        $totlalTime = 86400 * $no_days;
        $startTime  = \XF::$time - \XF::$time % $totlalTime;
        $endtime    = $startTime + $totlalTime;
        $options = \XF::options();
        $db = \XF::db();
        $query = "SELECT * FROM xf_thread AS thread , xf_user AS user 
        WHERE thread.user_id = user.user_id ";
        if(!empty($options->bw_node))
        {
            $query .= " AND thread.node_id NOT IN (".implode(",",$options->bw_node).") ";
        }

        $query .= " AND thread.post_date BETWEEN $startTime AND $endtime
        ORDER BY thread.view_count DESC LIMIT 1";
        $bestDetails = $db->fetchRow($query);
        if($bestDetails)
        {
            $user = \XF::em()->findOne('XF:User', ['user_id' , $bestDetails['user_id']]);
            return $user;
        }

        return false;
    }
    

    public function getActivePost($no_days)
    {
        $totlalTime = 86400 * $no_days;
        $startTime  = \XF::$time - \XF::$time % $totlalTime;
        $endtime    = $startTime + $totlalTime;
        $options = \XF::options();
        $db = \XF::db();
        $query = "SELECT * FROM xf_thread AS thread , xf_user AS user 
        WHERE thread.user_id = user.user_id ";
        if(!empty($options->bw_node))
        {
            $query .= " AND thread.node_id NOT IN (".implode(",",$options->bw_node).") ";
        }

        $query .= " AND thread.post_date BETWEEN $startTime AND $endtime
        ORDER BY thread.view_count DESC LIMIT 1";
        $bestDetails = $db->fetchRow($query);
        if($bestDetails)
        {
            $user = \XF::em()->findOne('XF:User', ['user_id' , $bestDetails['user_id']]);
            return $user;
        }

        return false;
    }



    public function rebuildBestThreadsWriter($no_days)
    {
        $cache = $this->getBestThreadsWriter($no_days);
        \XF::registry()->set('bw_bestThreadsWriter', $cache);
        return $cache;
    }

    public function rebuildBestPostsWriter($no_days)
    {
        $cache = $this->getBestPostsWriter($no_days);
        \XF::registry()->set('bw_bestPostsWriter', $cache);
        return $cache;
    }


    public function rebuildPobularPost($no_days)
    {
        $cache = $this->getPobularPost($no_days);
        \XF::registry()->set('bw_PobularPost', $cache);
        return $cache;
    }

    public function rebuildActivePost($no_days)
    {
        $cache = $this->getActivePost($no_days);
        \XF::registry()->set('bw_ActivePost', $cache);
        return $cache;
    }
}

